@@ -14,7 +14,7 @@
   Lesser General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with libcanberra. If not, If not, see
+  License along with libcanberra. If not, see
   <http://www.gnu.org/licenses/>.
 ***/
 
@@ -30,9 +30,9 @@
 
 typedef struct {
     guint signal_id;
+    gboolean arg1_is_set;
     GObject *object;
     GValue arg1;
-    gboolean arg1_is_set;
     GdkEvent *event;
 } SoundEventData;
 
@@ -80,7 +80,7 @@ static gboolean disabled = FALSE;
 
 static GQueue sound_event_queue = G_QUEUE_INIT;
 
-static int idle_id = 0;
+static guint idle_id = 0;
 
 static guint
     signal_id_dialog_response,
@@ -621,8 +621,8 @@ static void dispatch_sound_event(SoundEventData *d) {
                                     NULL);
     }
 
-    if (ret != CA_SUCCESS)
-        g_warning("Failed to play event sound: %s", ca_strerror(ret));
+/*     if (ret != CA_SUCCESS) */
+/*         g_warning("Failed to play event sound: %s", ca_strerror(ret)); */
 }
 
 static gboolean idle_cb(void *userdata) {
