@@ -14,7 +14,7 @@
   Lesser General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with libcanberra. If not, If not, see
+  License along with libcanberra. If not, see
   <http://www.gnu.org/licenses/>.
 ***/
 
@@ -528,7 +528,9 @@ int ca_context_play_full(ca_context *c, uint32_t id, ca_proplist *p, ca_finish_c
     ca_mutex_lock(c->mutex);
 
     ca_return_val_if_fail_unlock(ca_proplist_contains(p, CA_PROP_EVENT_ID) ||
-                                 ca_proplist_contains(c->props, CA_PROP_EVENT_ID), CA_ERROR_INVALID, c->mutex);
+                                 ca_proplist_contains(c->props, CA_PROP_EVENT_ID) ||
+                                 ca_proplist_contains(p, CA_PROP_MEDIA_FILENAME) ||
+                                 ca_proplist_contains(c->props, CA_PROP_MEDIA_FILENAME), CA_ERROR_INVALID, c->mutex);
 
     ca_mutex_lock(c->props->mutex);
     if ((t = ca_proplist_gets_unlocked(c->props, CA_PROP_CANBERRA_ENABLE)))
@@ -689,7 +691,7 @@ const char *ca_strerror(int code) {
         [-CA_ERROR_ACCESS] = "Access forbidden",
         [-CA_ERROR_IO] = "IO error",
         [-CA_ERROR_INTERNAL] = "Internal error",
-        [-CA_ERROR_DISABLED] = "Sounds disabled"
+        [-CA_ERROR_DISABLED] = "Sound disabled"
     };
 
     ca_return_val_if_fail(code <= 0, NULL);
