@@ -17,7 +17,7 @@
   Lesser General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with libcanberra. If not, If not, see
+  License along with libcanberra. If not, see
   <http://www.gnu.org/licenses/>.
 ***/
 
@@ -240,4 +240,10 @@ typedef void (*ca_free_cb_t)(void *);
 
 #define ca_streq(a, b) (strcmp((a),(b)) == 0)
 
+#ifdef __GNUC__
+#define CA_GCC_DESTRUCTOR __attribute__ ((destructor))
+#else
+#undef CA_GCC_DESTRUCTOR
+#endif
+
 #endif
