@@ -14,7 +14,7 @@
   Lesser General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with libcanberra. If not, If not, see
+  License along with libcanberra. If not, see
   <http://www.gnu.org/licenses/>.
 ***/
 
@@ -29,7 +29,7 @@
 #include "macro.h"
 
 char *ca_sprintf_malloc(const char *format, ...) {
-    int  size = 100;
+    size_t  size = 100;
     char *c = NULL;
 
     ca_assert(format);
@@ -49,12 +49,37 @@ char *ca_sprintf_malloc(const char *format, ...) {
 
         c[size-1] = 0;
 
-        if (r > -1 && r < size)
+        if (r > -1 && (size_t) r < size)
             return c;
 
         if (r > -1)    /* glibc 2.1 */
-            size = r+1;
+            size = (size_t) r+1;
         else           /* glibc 2.0 */
             size *= 2;
     }
 }
+
+#ifndef HAVE_STRNDUP
+char *ca_strndup(const char *s, size_t n) {
+    size_t n_avail;
+    char *p;
+
+    if (!s)
+        return NULL;
+
+    if (memchr(s, '\0', n)) {
+        n_avail = strlen(s);
+        if (n_avail > n)
+            n_avail = n;
+    } else
+        n_avail = n;
+
+    if (!(p = ca_new(char, n_avail + 1)))
+        return NULL;
+
+    memcpy(p, s, n_avail);
+    p[n_avail] = '\0';
+
+    return p;
+}
+#endif
