@@ -14,7 +14,7 @@
   Lesser General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with libcanberra. If not, If not, see
+  License along with libcanberra. If not, see
   <http://www.gnu.org/licenses/>.
 ***/
 
@@ -130,6 +130,22 @@ static pa_proplist *strip_prefix(pa_proplist *l, const char *prefix) {
     return l;
 }
 
+static void add_common(pa_proplist *l) {
+    ca_assert(l);
+
+    if (!pa_proplist_contains(l, CA_PROP_MEDIA_ROLE))
+        pa_proplist_sets(l, CA_PROP_MEDIA_ROLE, "event");
+
+    if (!pa_proplist_contains(l, CA_PROP_MEDIA_NAME)) {
+        const char *t;
+
+        if ((t = pa_proplist_gets(l, CA_PROP_EVENT_ID)))
+            pa_proplist_sets(l, CA_PROP_MEDIA_NAME, t);
+        else if ((t = pa_proplist_gets(l, CA_PROP_MEDIA_FILENAME)))
+            pa_proplist_sets(l, CA_PROP_MEDIA_NAME, t);
+    }
+}
+
 static int translate_error(int error) {
     static const int table[PA_ERR_MAX] = {
         [PA_OK]                       = CA_SUCCESS,
@@ -177,7 +193,10 @@ static void context_state_cb(pa_context *pc, void *userdata) {
         struct outstanding *out;
         int ret;
 
-        ret = translate_error(pa_context_errno(pc));
+        if (state == PA_CONTEXT_TERMINATED)
+            ret = CA_ERROR_DESTROYED;
+        else
+            ret = translate_error(pa_context_errno(pc));
 
         ca_mutex_lock(p->outstanding_mutex);
 
@@ -345,6 +364,7 @@ int driver_destroy(ca_context *c) {
 
         outstanding_free(out);
     }
+
     if (p->mainloop)
         pa_threaded_mainloop_free(p->mainloop);
 
@@ -502,7 +522,7 @@ static void stream_drain_cb(pa_stream *s, int success, void *userdata) {
 
     p = PRIVATE(out->context);
 
-    ca_assert(out->type = OUTSTANDING_STREAM);
+    ca_assert(out->type == OUTSTANDING_STREAM);
     ca_assert(out->clean_up);
 
     ca_mutex_lock(p->outstanding_mutex);
@@ -547,7 +567,7 @@ static void stream_write_cb(pa_stream *s, size_t bytes, void *userdata) {
 
         ca_assert(rbytes <= bytes);
 
-        if ((ret = pa_stream_write(s, data, rbytes, free, 0, PA_SEEK_RELATIVE)) < 0) {
+        if ((ret = pa_stream_write(s, data, rbytes, ca_free, 0, PA_SEEK_RELATIVE)) < 0) {
             ret = translate_error(ret);
             goto finish;
         }
@@ -624,6 +644,7 @@ int driver_play(ca_context *c, uint32_t id, ca_proplist *proplist, ca_finish_cal
     int try = 3;
     int ret;
     pa_operation *o;
+    char *sp;
 
     ca_return_val_if_fail(c, CA_ERROR_INVALID);
     ca_return_val_if_fail(proplist, CA_ERROR_INVALID);
@@ -650,15 +671,11 @@ int driver_play(ca_context *c, uint32_t id, ca_proplist *proplist, ca_finish_cal
     if ((ret = convert_proplist(&l, proplist)) < 0)
         goto finish;
 
-    if (!(n = pa_proplist_gets(l, CA_PROP_EVENT_ID))) {
-        ret = CA_ERROR_INVALID;
-        goto finish;
-    }
-
-    if (!(name = ca_strdup(n))) {
-        ret = CA_ERROR_OOM;
-        goto finish;
-    }
+    if ((n = pa_proplist_gets(l, CA_PROP_EVENT_ID)))
+        if (!(name = ca_strdup(n))) {
+            ret = CA_ERROR_OOM;
+            goto finish;
+        }
 
     if ((vol = pa_proplist_gets(l, CA_PROP_CANBERRA_VOLUME))) {
         char *e = NULL;
@@ -681,54 +698,74 @@ int driver_play(ca_context *c, uint32_t id, ca_proplist *proplist, ca_finish_cal
         }
 
     strip_prefix(l, "canberra.");
+    add_common(l);
 
     if ((ret = subscribe(c)) < 0)
         goto finish;
 
-    for (;;) {
-        pa_threaded_mainloop_lock(p->mainloop);
+    if (name) {
+
+        /* Ok, this sample has an event id, let's try to play it from the cache */
+
+        for (;;) {
+            pa_threaded_mainloop_lock(p->mainloop);
+
+            /* Let's try to play the sample */
+            if (!(o = pa_context_play_sample_with_proplist(p->context, name, c->device, v, l, play_sample_cb, out))) {
+                ret = translate_error(pa_context_errno(p->context));
+                pa_threaded_mainloop_unlock(p->mainloop);
+                goto finish;
+            }
+
+            while (pa_operation_get_state(o) != PA_OPERATION_DONE)
+                pa_threaded_mainloop_wait(p->mainloop);
+
+            pa_operation_unref(o);
 
-        /* Let's try to play the sample */
-        if (!(o = pa_context_play_sample_with_proplist(p->context, name, c->device, v, l, play_sample_cb, out))) {
-            ret = translate_error(pa_context_errno(p->context));
             pa_threaded_mainloop_unlock(p->mainloop);
-            goto finish;
+
+            /* Did we manage to play the sample or did some other error occur? */
+            if (out->error != CA_ERROR_NOTFOUND)
+                goto finish;
+
+            /* Hmm, we need to play it directly */
+            if (cache_control != CA_CACHE_CONTROL_PERMANENT)
+                break;
+
+            /* Don't loop forever */
+            if (--try <= 0)
+                break;
+
+            /* Let's upload the sample and retry playing */
+            if ((ret = driver_cache(c, proplist)) < 0)
+                goto finish;
         }
-
-        while (pa_operation_get_state(o) != PA_OPERATION_DONE)
-            pa_threaded_mainloop_wait(p->mainloop);
-
-        pa_operation_unref(o);
-
-        pa_threaded_mainloop_unlock(p->mainloop);
-
-        /* Did we manage to play the sample or did some other error occur? */
-        if (out->error != CA_ERROR_NOTFOUND)
-            goto finish;
-
-        /* Hmm, we need to play it directly */
-        if (cache_control != CA_CACHE_CONTROL_PERMANENT)
-            break;
-
-        /* Don't loop forever */
-        if (--try <= 0)
-            break;
-
-        /* Let's upload the sample and retry playing */
-        if ((ret = driver_cache(c, proplist)) < 0)
-            goto finish;
     }
 
     out->type = OUTSTANDING_STREAM;
 
     /* Let's stream the sample directly */
-    if ((ret = ca_lookup_sound(&out->file, &p->theme, c->props, proplist)) < 0)
+    if ((ret = ca_lookup_sound(&out->file, &sp, &p->theme, c->props, proplist)) < 0)
         goto finish;
 
+    if (sp)
+        if (!pa_proplist_contains(l, CA_PROP_MEDIA_FILENAME))
+            pa_proplist_sets(l, CA_PROP_MEDIA_FILENAME, sp);
+
+    ca_free(sp);
+
     ss.format = sample_type_table[ca_sound_file_get_sample_type(out->file)];
-    ss.channels = ca_sound_file_get_nchannels(out->file);
+    ss.channels = (uint8_t) ca_sound_file_get_nchannels(out->file);
     ss.rate = ca_sound_file_get_rate(out->file);
 
+    if (!name) {
+        if (!(n = pa_proplist_gets(l, CA_PROP_MEDIA_NAME)))
+            if (!(n = pa_proplist_gets(l, CA_PROP_MEDIA_NAME)))
+                n = "libcanberra";
+
+        name = ca_strdup(n);
+    }
+
     pa_threaded_mainloop_lock(p->mainloop);
 
     if (!(out->stream = pa_stream_new_with_proplist(p->context, name, &ss, NULL, l))) {
@@ -864,6 +901,7 @@ int driver_cache(ca_context *c, ca_proplist *proplist) {
     ca_cache_control_t cache_control = CA_CACHE_CONTROL_PERMANENT;
     struct outstanding *out;
     int ret;
+    char *sp;
 
     ca_return_val_if_fail(c, CA_ERROR_INVALID);
     ca_return_val_if_fail(proplist, CA_ERROR_INVALID);
@@ -910,13 +948,20 @@ int driver_cache(ca_context *c, ca_proplist *proplist) {
     strip_prefix(l, "canberra.");
     strip_prefix(l, "event.mouse.");
     strip_prefix(l, "window.");
+    add_common(l);
 
     /* Let's stream the sample directly */
-    if ((ret = ca_lookup_sound(&out->file, &p->theme, c->props, proplist)) < 0)
+    if ((ret = ca_lookup_sound(&out->file, &sp, &p->theme, c->props, proplist)) < 0)
         goto finish;
 
+    if (sp)
+        if (!pa_proplist_contains(l, CA_PROP_MEDIA_FILENAME))
+            pa_proplist_sets(l, CA_PROP_MEDIA_FILENAME, sp);
+
+    ca_free(sp);
+
     ss.format = sample_type_table[ca_sound_file_get_sample_type(out->file)];
-    ss.channels = ca_sound_file_get_nchannels(out->file);
+    ss.channels = (uint8_t) ca_sound_file_get_nchannels(out->file);
     ss.rate = ca_sound_file_get_rate(out->file);
 
     pa_threaded_mainloop_lock(p->mainloop);
@@ -930,7 +975,7 @@ int driver_cache(ca_context *c, ca_proplist *proplist) {
     pa_stream_set_state_callback(out->stream, stream_state_cb, out);
     pa_stream_set_write_callback(out->stream, stream_write_cb, out);
 
-    if (pa_stream_connect_upload(out->stream, ca_sound_file_get_size(out->file)) < 0) {
+    if (pa_stream_connect_upload(out->stream, (size_t) ca_sound_file_get_size(out->file)) < 0) {
         ret = translate_error(pa_context_errno(p->context));
         pa_threaded_mainloop_unlock(p->mainloop);
         goto finish;
